#include <ArduinoJson.h>

int waitingTime = 30000; // 等候 30 秒的回覆.

String Check_RevData()        // 讀取收到的每一字元資料 ，彙整成一個字串
{
 String data= "";	
 while (BC26_Serial.available())
 {
  data=BC26_Serial.readStringUntil('\n');
  //Serial.println(data);
  break; 
 }
 return data;  
}

byte Send_ATcommand(String msg,byte stepnum) // 傳送 AT command ，並加以判斷
{
 String Showmsg,C_temp;
 Serial.println(msg);
 BC26_Serial.println(msg);
 Showmsg=Check_RevData();
 //Serial.println(Showmsg);
 long StartTime=millis();
 switch (stepnum)
  {
    case 0:  // Reset BC26
         C_temp="+IP:";
         break;    
    case 1:         // Other Data
         C_temp="OK";
         break; 
    case 2:         // Check IPAddress 
         C_temp="+CGPADDR:"; 
         break; 
    case 3:         // QGACT
         C_temp="+QGACT:";
         break;  
    case 4:
         C_temp="+CGATT: 1";
         break;           
    case 10:  // build MQTT Server
         C_temp="+QMTOPEN: 0,0";
         break;        
    case 11: // Connect to MQTT server by username and password
         C_temp="+QMTCONN: 0,0,0";
         break;  
    case 12: // Publisher MQTT Data 
         C_temp="+QMTPUB: 0,0,0";
         break;        
    case 13: // Sub MQTT Data 
         C_temp="+QMTSUB: 0,1,0";
         break;
    case 15: // Close MQTT Data 
         C_temp="+QMTCLOSE: 0,0";
         break; 
    case 20: // HTTP open     
         C_temp="+QIOPEN: 0,0";
         break;
    case 21: // HTTP Close     
         C_temp="CLOSE OK";
         break;                                    
  }
  while (!Showmsg.startsWith(C_temp))
  {
   Showmsg=Check_RevData();
   if (Showmsg.startsWith("+")) Serial.println(Showmsg);
   if ((StartTime+waitingTime) < millis()) return stepnum;
   //delay (50);
  }          
  return 99;
}

bool BC26init()  // 初始化 BC26
{
 Send_ATcommand("AT+QBAND=1,8",1);
 Send_ATcommand("AT+QGACT=1,1,\"apn\",\"internet.iot\"",3);
 Send_ATcommand("AT+QCGDEFCONT=\"IP\",\"internet.iot\"",1);
 if (Send_ATcommand("AT+QRST=1",0)==99) return true;
 return false;
}

bool connect_MQTT(String Serverx,String port,String user,String pass) // 建立 MQTT 連線通道
{
 String S_temp;
 String S_imei;
 String Send_check="";
 long StartTime=millis();
 S_temp="AT+CGSN=1";
 //Serial.println(S_temp);
 BC26_Serial.println(S_temp);
 while (!Send_check.startsWith("+CGSN:"))
 {
  //Serial.println("Wait ...");
  Send_check=Check_RevData();
  //if (Send_check.startsWith("+")) Serial.println(Send_check);  
  if ((StartTime+waitingTime) < millis()) return false;
  //delay (50);
 }
 S_imei=Send_check.substring(7,22);
 //Serial.println(S_imei);
  
 S_temp = "\"" + Serverx + "\"" + "," + port;
 S_temp="AT+QMTOPEN=0," + S_temp;
 //Serial.println(S_temp);
 if (Send_ATcommand(S_temp,10)==99)
 {
  S_temp= "\"" + S_imei + "\"" + "," + "\"" + user + "\"" + "," + "\"" + pass + "\"";
  S_temp="AT+QMTCONN=0," + S_temp;
  //Serial.println(S_temp);
  delay(50);
  if (Send_ATcommand(S_temp,11)==99)
  {
  // delay(50);
   return true;
  }
  S_temp="AT+QMTCLOSE=0"; 
  Send_ATcommand(S_temp,15); 
 } 
 return false;
}

bool Publish_MQTT(String topic, String message) // 發佈資料
{ 
 String S_temp;
 S_temp = "\"" + topic + "\"" + "," + message ;
 S_temp = "AT+QMTPUB=0,0,0,1," + S_temp ;
 //Serial.println(S_temp);
 if (Send_ATcommand(S_temp,12)!=99) return false; 
 //delay(100);
 return true;
}

bool Sub_MQTT(String topic) // 訂閱資料
{
 String S_temp;
 S_temp = "\"" + topic + "\"" + "," + "0";
 S_temp = "AT+QMTSUB=0,1," + S_temp; 
 //Serial.println(S_temp);
 if (Send_ATcommand(S_temp,13)!= 99) return false;  
 //delay(100); 
 return true; 
}

String Sub_MQTT_Data (String topic)
{
 String S_temp;
 String S_SubData;
 String Send_check="";
 long StartTime=millis();
 S_temp = "\"" + topic + "\"" + "," + "2";
 S_temp = "AT+QMTSUB=0,1," + S_temp;
 Serial.println(S_temp);
 BC26_Serial.println(S_temp);
 while (!Send_check.startsWith("+QMTRECV"))
 {
  //Serial.println("Wait ...");  
  //delay (100);
  Send_check=Check_RevData();
  if (Send_check.startsWith("+")) Serial.println(Send_check);
  if (Send_check.startsWith("+QMTSUB: 0,1,0")) break;
  if ((StartTime+waitingTime) < millis()) return "ERROR";
 }
 S_SubData=Send_check.substring(15+topic.length()+3,15+topic.length()+3+2);
 //Serial.println(S_SubData);
 return S_SubData;
}

bool Close_MQTT() // 關閉連線
{ 
 String S_temp;
 S_temp="AT+QMTCLOSE=0";
 if (Send_ATcommand(S_temp,15)!=99) return false; 
 //delay(100);
 return true;
}

bool Connect_HTTP_Server(String S_name) // 建立 HTTP 連線通道
{
 String S_temp="";
 S_temp="AT+QIOPEN=1,0,\"TCP\",\"" + S_name + "\",80,0,0";
 //Serial.println(S_temp);
 if (Send_ATcommand(S_temp,20)!=99) return false;
 //Send_ATcommand("AT+QICFG=\"viewmode\",1",4);
 //delay(50);
 return true;  
}

bool Send_Data_IFTTT(String SHost,String SKey,String SEvent,String v1,String v2,String v3) // 傳送資料給 IFTTT 與 關閉連線
{
 String Send_check=""; 
 String S_tempdata=""; 
 String S_temphost="";
 long StartTime=millis();
 S_tempdata="POST /trigger/" + SEvent +"/with/key/"+SKey+"?value1="+v1+"&value2="+v2+"&value3="+v3+" HTTP/1.1";
 S_temphost="HOST: " + SHost ;
 Serial.println("AT+QISEND=0");
 BC26_Serial.println("AT+QISEND=0");
 delay(50);
 BC26_Serial.println(S_tempdata);
 BC26_Serial.println(S_temphost);
 BC26_Serial.println();
 BC26_Serial.print("\x1A");
 while (!Send_check.startsWith("+QIURC: \"recv\",0"))
 {
  //Serial.println("Wait ...");  
  
  Send_check=Check_RevData();
  if (Send_check.startsWith("+")) Serial.println(Send_check);  
  if ((StartTime+waitingTime) < millis()) return false;
  //delay (50);
 }
 //delay (100);
 return true; 
}

bool Close_HTTP_Server() // 關閉 HTTP 連線通道
{
 //Serial.println("AT+QICLOSE=0"); 
 Send_ATcommand("AT+QICLOSE=0",21);
 //delay(100);
 return true;  
}

String JSON_DEC_data (String input,String findData)
{
 int index = input.indexOf(',');
 int x = input.substring(0, index).toInt();
 String json = input.substring(index + 1, input.length());
 //Serial.println(json);
 index = json.indexOf(':');
 x = json.substring(0, index).toInt();
 json = json.substring(index + 1, json.length());
 //Serial.println(json);
 DynamicJsonDocument doc(1024);
 deserializeJson(doc, json);
 JsonObject obj = doc.as<JsonObject>();
 return obj[findData];
}

bool Sub_Ideaschain(String attrestopic)
{
 String S_temp;
 S_temp="\""+ attrestopic + "\"" + "," + "1";
 S_temp="AT+QMTSUB=0,1," + S_temp;  // Qos 0 
 Serial.println(S_temp);
 BC26_Serial.println(S_temp);
 //delay (2000);
 return true; 
}

String Get_Publish_MQTT(byte mode,String attreqtopic , String message) // 取得 Ideaschain 屬性資料
{
 String Showmsg; 
 String S_temp,T_temp;
 //delay (1000);
 if (mode==0) T_temp="sharedKeys";
 if (mode==1) T_temp="clientKeys";
 S_temp="\"" + attreqtopic + "\"" + "," + "\"{\"" + T_temp + "\":\"" + message + "\"}\"";
 S_temp="AT+QMTPUB=0,0,0,0," + S_temp; 
 Serial.println(S_temp);
 BC26_Serial.println(S_temp);
 Showmsg=Check_RevData();
 long StartTime=millis();
 while (!Showmsg.startsWith("+QMTRECV:"))
 {  
  Showmsg=Check_RevData(); 
  //Serial.println(Showmsg); 
  if (Showmsg.length()>50) break;  
  if ((StartTime+waitingTime) < millis()) return "error";
  //delay(50);
 }
 //Serial.println(Showmsg);  
 //delay (100);  
 return JSON_DEC_data (Showmsg,message);
}
